# Generated by roxygen2: do not edit by hand

S3method(print,solnp_kkt_summary)
export(csolnp)
export(csolnp_ms)
export(gosolnp)
export(kkt_diagnose)
export(solnp)
export(solnp_problem_suite)
export(solnp_problems_table)
export(solnp_standardize_problem)
export(startpars)
importFrom(Rcpp,evalCpp)
importFrom(future.apply,future_lapply)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(numDeriv,jacobian)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,parLapply)
importFrom(stats,optim)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(truncnorm,rtruncnorm)
importFrom(utils,tail)
useDynLib(Rsolnp, .registration=TRUE)
